<?php
require_once("cwfUI.inc.php");
require_once("ViewDataUI.inc.php");

class AddRegistrationUI extends ViewDataUI {
  protected static $accessRight = "";
  protected static $dataClass = "";
  protected $view;
  
  /////////////////////////////////////////////////////////////////////////////////////////////////
  // initialize
  public function __construct(DbConnection $DB) {
    $this->initialize($DB);
  }
  
  protected function initialize(DbConnection $DB) {
    $this->view = new View();
    $this->columns = array();
    $this->showSelect = true;
    $this->showSelectAll = false;
    
    $this->eventId = null;
    if(isset($_SESSION['event']))
      $this->eventId = $_SESSION['event']->event_id;
    
    $this->view->columns = array("user_id", "familyname", "givenname", "email");
    $this->view->edit_booking_check = 0;
    $this->view->edit_user_data = 0;
    
    $this->title = _("Add registrations");
    
    $this->header = '  <script src="'.CWF_HTTP_PATH.'k_db/js/kdb_view.js" type="text/javascript"></script>'."\n";
    $this->header .= '  <script src="'.CWF_HTTP_PATH.'k_db/js/kdb_add_registration.js" type="text/javascript"></script>'."\n";
    $this->header .= '  <link rel="stylesheet" href="'.CWF_HTTP_PATH.'k_db/css/view.css" type="text/css">';
    $this->header .= "  <script type=\"text/javascript\">\n";
    $this->header .= "    var uiClassName = '".get_class($this)."';\n";
    $this->header .= "    $(document).ready(function(){\n";
    $this->header .= "      setupTableEvents(0);\n";
    $this->header .= "      $('#kdb_form_user').validate({\n";
    $this->header .= "        errorClass: 'invalid',\n";
    $this->header .= self::getValidationRules('User')->getValidationRulesJQuery("        ");
    $this->header .= "      });\n";
    $this->header .= "      $('#kdb_form_view').validate({\n";
    $this->header .= "        errorClass: 'invalid',\n";
    $this->header .= self::getValidationRules('UserBooking')->getValidationRulesJQuery("        ");
    $this->header .= "      });\n";
    $this->header .= "    });\n";
    $this->header .= "  </script>\n";
    
    $this->sql = $this->getSQL($DB);
  }
  
  /////////////////////////////////////////////////////////////////////////////////////////////////
  // handle data
  public function getPostBackXML(DbConnection $DB, $action) {
    $xml = "";
    switch ($action) {
      case "saveasnew":
      case "save":
        //$xml = $this->getNewValuesXML($this->data->getPostBackData());
        if(get_class($this->data)=="User") {
          $newUser = array();
          $str = "  <td style=\"white-space:nowrap;\">".
              "<button onClick=\"kdbUserDel($(this));\" type=\"button\"><img src=\"../../k_db/img/icons/delete.png\" alt=\"New\" height=\"12\" width=\"12\"></button>".
              "<input type=\"hidden\" id=\"user_id[{$this->data->user_id}]\" name=\"user_id[{$this->data->user_id}]\" value=\"{$this->data->user_id}\"></td>\n";
          foreach ($this->view->columns as $column)
            $newUser[$column] = $this->data->$column;
          
          $xml .= "  <appendHTMLElement id=\"table#selected_users tbody\"><![CDATA[\n";
          $xml .= "<tr>\n  {$str}  <td>".implode("</td>\n  <td>", $newUser)."</td>\n</tr>\n";
          $xml .= "]]>\n";
          $xml .= "  </appendHTMLElement>\n";
        }
        else if(get_class($this->data)=="UserBooking") {
          $xml .= parent::getPostBackXML($DB, "updateView");
        }
        break;
      default:
        $xml .= parent::getPostBackXML($DB, $action);
    }
    return $xml;
  }
  
  static public function getValidationRules($form=null) {
    $rules = array();
    $messages = array();
    switch($form) {
      case "User":
        return UserUI::getValidationRules();
        break;
      case "UserBooking":
        return UserBookingUI::getValidationRules();
        break;
    }
    
    $validationRules = new FormValidation($rules, $messages);
    return $validationRules;
  }
  
  /////////////////////////////////////////////////////////////////////////////////////////////////
  // Database operations
  public function saveForm(DbConnection $DB) {
    switch ($_POST['formName']) {
      case "kdb_form_user":
        $this->data = new User(self::getValidationRules("User"));
        if($this->data->fromArray($_POST, true)) {
          list($errno, $error) = $this->data->writeData($DB, true);
          if ($errno) {
            $this->error = $error;
            return 0;
          }

          if(ALLOW_USER_LOGIN) {
            $confirmationText = KDB_CONFIRMATION_EMAIL;
            $confirmationText = $this->data->getConfirmationText($DB, $confirmationText, 1);
            unset($_SESSION['reg_passwd']);
            if(strstr($confirmationText, "\$FORM_DATA\$")) {
              $confirmationText = str_replace("\$FORM_DATA\$", $this->data->getFormData($DB), $confirmationText);
            }
            KDbTools::sendConfirmation($this->data->email, $confirmationText, ORGANISATION_NAME, ROOT_EMAIL);
          }
          return 1;
        }
        $this->error = _("Validation failed.");
        return 0;
        break;
      case "kdb_form_view":
        $this->data = new UserBooking(self::getValidationRules("UserBooking"));
        if(isset($_POST['user_id']) && count($_POST['user_id'])>0) {
          $this->data->setEventId($DB, $_SESSION['event']->event_id);
          if($this->data->fromArray($_POST)) {
            foreach ($_POST['user_id'] as $idx => $value) {
              $this->data->setUserId($value);
              $this->data->unsetBookingId();
              list($errno, $error) = $this->data->writeData($DB);
              if ($errno) 
                $this->error = $error;
              else {
                $user = new User();
                $user->readData($DB, array("user_id"=>"$value"));
                if($this->data->isWaitlisted() && $_SESSION['event']->confirmation_email_wl)
                  $confirmationText = $_SESSION['event']->confirmation_email_wl;
                else
                  $confirmationText = $_SESSION['event']->confirmation_email;
                $confirmationText = $user->getConfirmationText($DB, $confirmationText);
                $confirmationText = $this->data->getConfirmationText($DB, $confirmationText);
                if(strstr($confirmationText, "\$FORM_DATA\$")) {
                  $confirmationText = str_replace("\$FORM_DATA\$", $this->data->getFormData($DB), $confirmationText);
                }
                KDbTools::sendConfirmation($user->email, $confirmationText, $_SESSION['event']->event_title, $_SESSION['event']->event_email, $_SESSION['event']->reg_email);
              }
            }
            return 1;
          }
          else {
            $this->error = _("Validation failed.");
            return 0;
          }
        }
        $this->error = _("No users selected.");
        return 0;
        break;
    }
  }
  
  public function deleteData(DbConnection $DB, $id) {
    
  }
  
  function getSQL(DbConnection $DB) {
    list($whereClause, $orderBy, $limit) = $this->prepareSQL($DB);
    
    $sql = "SELECT ".implode(",\n       ", $this->getColumns("forSelect", true))."\n".
           "FROM users u\n".
           "  LEFT JOIN user_booking ub ON (ub.user_id=u.user_id AND ub.deleted=0 AND ub.event_id='{$_SESSION['event']->event_id}')\n".
           "  LEFT JOIN user_booking_check ubc ON ubc.booking_id=ub.booking_id\n".
           "WHERE u.deleted=0\n".
           "GROUP BY u.user_id\n".
           (count($whereClause)>0 ? "HAVING ".implode("\n  AND ", $whereClause)."\n" : "").
           "ORDER BY {$orderBy}\n".
           "LIMIT 10;";

    return $sql;
  }
  
  /////////////////////////////////////////////////////////////////////////////////////////////////
  // output
    
  protected function getViewTableHeader() {
    $str = "<thead>\n";
  
    //print table header
    $str .= "<tr>\n";
      
    foreach ($this->columns as $column) {
      $str .= $this->getColumnHeader($column);
    }
  
    $str .= "</tr>\n";
    $str .= "</thead>\n";
  
    return $str;
  }
  
  protected function getViewTableCell($column, $row) {
    $str = "";
    if($column['columnName']=="selectUser") {
      if($row['booking_id'] && !$_SESSION['event']->allow_multiple_reg)
        $str .= "  <td style=\"white-space:nowrap;\">".
                "<input type=\"hidden\" id=\"user_id[{$row['user_id']}]\" name=\"user_id[{$row['user_id']}]\" value=\"{$row['user_id']}\" disabled></td>\n";
      else
        $str .= "  <td style=\"white-space:nowrap;\">".
                "<button onClick=\"kdbUserAdd($(this));\" type=\"button\"><img src=\"../../k_db/img/icons/add.png\" alt=\"New\" height=\"12\" width=\"12\"></button>".
                "<input type=\"hidden\" id=\"user_id[{$row['user_id']}]\" name=\"user_id[{$row['user_id']}]\" value=\"{$row['user_id']}\" disabled></td>\n";
    }
    else
      $str .= parent::getViewTableCell($column, $row);
    
    return $str;
  }
  
  public function getViewTableBody(DbConnection $DB) {
    if (!isset($_SESSION['filter']) || count($_SESSION['filter'])==0) {
      $str = "<tbody>\n";
      $str .= "<tr><td colspan=\"".(count($this->columns))."\">"._("No filter applied.")."</td></tr>\n";
      $str .= "</tbody>\n";
    }
    else
      $str = parent::getViewTableBody($DB);
    
    return $str;
  }
  
  public function getHTML(DbConnection $DB) {
?>
<form id="kdb_form_view" action="" method="post" enctype="multipart/form-data">

<div class="form_controls_top">
<p>
  <input type="submit" id="kdb_reg_users" name="kdb_reg_users" value="<?=_("Book selected users")?>" onClick="kdbSaveEntryAddReg('AddRegistrationUI', 'save', 'kdb_form_view'); return false;">
  <input type="submit" id="kdb_add_user" name="kdb_add_user" value="<?=_("Add user")?>" onClick="kdbUpdateViewDialog(); return false;">
</p>
</div>

<?php
    echo "<h3>"._("Registered users")."</h3>\n";
    if(!isset($_SESSION['event']))
      echo "<p>"._("No event selected.")."</p>\n";
    else
      echo $this->getViewTable($DB);
    
    echo "<h3>"._("Selected users")."</h3>\n";
    echo "<table id=\"selected_users\" class=\"view\">\n";
    echo "<thead>\n";
    
    //print table header
    echo "<tr>\n";
    foreach ($this->columns as $column) {
      echo "  <th><div class=\"tableheader\">".(!$column['displayName'] ? "&nbsp;" : $column['displayName'])."</div></th>\n";
    }
    echo "</tr>\n";
    echo "</thead>\n";
    echo "<tbody>\n";
    echo "<tr id=\"no_user_selected\"><td colspan=\"".(count($this->columns))."\">"._("No users selected.")."</td></tr>\n";
    echo "</tbody>\n";
    echo "</table>\n";
?>
<?php 
    $bookingUI = new UserBookingUI($DB);
    $bookingUI->getHTMLView($DB, false, 1); 
?>

<div class="form_controls_bottom">
<p>
  <input type="submit" id="kdb_reg_users2" name="kdb_reg_users" value="<?=_("Book selected users")?>" onClick="kdbSaveEntryAddReg('AddRegistrationUI', 'save', 'kdb_form_view'); return false;">
  <input type="submit" id="kdb_add_user2" name="kdb_add_user" value="<?=_("Add user")?>" onClick="kdbUpdateViewDialog(); return false;">
</p>
</div>

</form>

<span id="tt_filter" class="tooltip below"><?= _("Wildcards:<br><u>*</u> any number of characters,<br><u>?</u> any single character.<br>Regular expressions must be enclosed in '/' (e.g. '/[a-z]+/').") ?></span>

<div id="dialog_view" class="dialog_container">
<div class="dialog_columns viewui">
  <div class="tab active" onClick="$('#dialog_view .tab').removeClass('active'); $(this).addClass('active'); $('#dialog_view .dialog').hide();  $('#dialog_view #dialog_user').show();">Benutzer</div>
  <div class="close_dialog" onClick="$('#dialog_view').hide();"></div>
  <div class="dialog" id="dialog_user">
<?php
    $userUI = new UserUI($DB);
    $userUI->getHTMLAddReg($DB); 
?>
  </div>
</div>
</div>
<?php
  }
}

?>
