<?php
require_once("icwfData.inc.php");
require_once("FormValidation.inc.php");

class UserBookingCheck implements icwfData {
  //private static $accessRight = "edit_users";
  private static $noExport = array();
  private static $notInArray = array("validation");
  private $validation;
  
  private $booking_id;
  private $payment_received;
  private $uploads_checked;
  private $has_checked_in;
  private $discount;
  private $comment;

  /////////////////////////////////////////////////////////////////////////////////////////////////
  // initialize
  private function initialize() {
    foreach ($this as $key => $value)
      $this->$key = null;
  }
  
  public function __construct(FormValidation $validation=null) {
    $this->initialize();
    $this->validation = $validation;
  }
  
  /////////////////////////////////////////////////////////////////////////////////////////////////
  // get values
  public function __get($name) {
    if($name == "id") $name = "booking_id";
    if (array_key_exists($name, get_class_vars(get_class())) && !in_array($name, self::$noExport))
      return $this->$name;
      
    return null;
  }

  public function getIDName() {
    return "booking_id";
  }
  
  public function toArray() {
    $array = array();
    foreach ($this as $key => $value) {
      if(!in_array($key, self::$notInArray)) {
        $array[$key] = $value;
      }
    }
    return $array;
  }
  
  public function getPostBackData() {
    return array($this->getIDName()=>$this->id);
  }
  
  /////////////////////////////////////////////////////////////////////////////////////////////////
  // set values
  public function setBookingId($bookingId) {
    $this->booking_id = $bookingId;
  }
  
  public function setValidation(FormValidation $validation) {
    $this->validation = $validation;
  }
  
  public function fromArray($array, $idx=0) {
    foreach ($this as $key => $value) {
      switch ($key) {
        case "validation":
          break;
        
        case "booking_id":
          if(isset($array[$key][$idx]) && preg_match("~^[0-9]+$~", $array[$key][$idx]))
            $this->$key = $array[$key][$idx];
          else
            $this->$key = null;
          break;
          
        case "discount":
          if(isset($array[$key][$idx]) && preg_match("~^-?[0-9]+$~", $array[$key][$idx]))
            $this->$key = $array[$key][$idx];
          else
            $this->$key = null;
          break;
        
        case "comment":
          if(isset($array[$key][$idx]))
            $this->$key = $array[$key][$idx];
          else
            $this->$key = null;
          break;
          
        default:
          $this->$key = ((isset($array[$key][$idx]) && $array[$key][$idx]) ? 1 : 0);
          break;
      }
    }
    return true;
    //return $this->validation->validate($array);
  }
  
  /////////////////////////////////////////////////////////////////////////////////////////////////
  // database operations - read
  public function readData(DbConnection $DB, $conditions=null) {
    if(is_array($conditions)) {
      $whereConditions = array();
      foreach ($conditions as $key => $value)
        if(array_key_exists($key, get_class_vars(get_class())))
          $whereConditions[] = "$key=$value";
      $conditions =  implode(" AND ", $whereConditions);
    }
    $result = $DB->query("SELECT * FROM user_booking_check WHERE {$conditions};");
    
    if(!$DB->errno() && $DB->num_rows($result)>0) {
      $booking = $DB->fetch_array($result);
      foreach ($booking as $property => $value) {
        if(property_exists($this, $property)) {
          switch ($property) {
            default:
              $this->$property = $value;
              break;
          }
        }
      }
      return array(0, "");
    }
    else
      return array($DB->errno(), $DB->error());
  }
  
  static public function readList(DbConnection $DB) {
    return array();
  }
  
  /////////////////////////////////////////////////////////////////////////////////////////////////
  // database operations - write
  public function writeData(DbConnection $DB) {
    if(!isset($this->booking_id) || !$this->booking_id) return array(1, _("ID not set."));

    $result = $DB->query("UPDATE user_booking_check
        SET payment_received='{$this->payment_received}',
            uploads_checked='{$this->uploads_checked}',
            has_checked_in='{$this->has_checked_in}',
            discount='{$this->discount}',
            comment='{$this->comment}'
        WHERE booking_id='{$this->booking_id}';");
    if ($DB->affected_rows()<=0)
      $result = $DB->query("INSERT INTO user_booking_check(
          booking_id,
          payment_received,
          uploads_checked,
          has_checked_in,
          discount,
          comment
        ) VALUES(
          '{$this->booking_id}',
          '{$this->payment_received}',
          '{$this->uploads_checked}',
          '{$this->has_checked_in}',
          '{$this->discount}',
          '{$this->comment}'
        );");
    
    return array($DB->errno(), $DB->error());
  }

  static public function deleteData(DbConnection $DB, $id) {
    if(preg_match("~^[0-9]+$~", $id))
      $condition = "booking_id='{$id}'";
    else
      $condition = $id;
      
    $DB->query("DELETE FROM user_booking_check WHERE $condition;");
    
    if(!$DB->errno() && $DB->affected_rows()>0)
      return array(0, "");
    else 
      return array($DB->errno(), $DB->error());
  }

  /////////////////////////////////////////////////////////////////////////////////////////////////
  // database operations - misc
  static public function createTable(DbConnection $DB) {
    $result = $DB->query("
      CREATE TABLE user_booking_check (
        booking_id int unsigned NOT NULL PRIMARY KEY,
        payment_received tinyint,
        uploads_checked tinyint,
        has_checked_in tinyint,
        discount int NOT NULL,
        comment text
      );");
    if ($result) { echo _("&nbsp;&nbsp;- creating table").": 'user_booking_check'.<br>\n"; }
    else { echo _("&nbsp;&nbsp;<b>- Error when creating table</b>")." 'user_booking_check'<br>\n"; $error = 1; }
  }

  /////////////////////////////////////////////////////////////////////////////////////////////////
  // output of formatted data
  static public function getProperties($property=null) {
    $properties = array(
      'payment_received' => _("Payment received"),
      'uploads_checked' => _("Uploads checked"),
      'has_checked_in' => _("Has checked in"),
      'discount' => _("Discount"),
      'comment' => _("Comment")
    );
    
    if(is_array($property)) {
      $newProperties = array();
      foreach($property as $id=>$p)
        if(isset($properties[$p])) $newProperties[$id] = $properties[$p];
      return $newProperties;
    }
    else if($property) {
      $property = preg_replace('~^([^.]+\.)?([^.]+)$~', '\2', $property);
      return ((isset($properties[$property])) ? $properties[$property] : null);
    }
    else
      return $properties;
  }

  static public function getPropertySelect($columnNames, $editable, $addTableName="") {
    if(is_array($columnNames)) {
      foreach($columnNames as &$columnName)
        $columnName = self::getPropertySelect($columnName, $editable, $addTableName);
    }
    else {
      $property = preg_replace('~^([^.]+\.)?([^.]+)$~', '\2', $columnNames);
      if(!array_key_exists($property, get_class_vars(get_class()))) return $columnNames;

      $columnNames = $addTableName . $property;
      if(!$editable) {
        switch ($property) {
          case 'payment_received':
          case 'uploads_checked':
          case 'has_checked_in':
            $columnNames = "IF({$columnNames}>0, '"._("yes")."', '"._("no")."') {$property}";
            break;
        }
      }
    }
    return $columnNames; 
  }
  
  static public function getFormElement($property, $formElementName, $value) {
    switch($property) {
      case 'payment_received':
      case 'uploads_checked':
      case 'has_checked_in':
        return "<input type=\"checkbox\" id=\"{$formElementName}\" name=\"{$formElementName}\" value=\"1\"".($value ? " checked" : "").">";
        break;
      case 'discount':
      case 'comment':
        return "<input type=\"text\" id=\"{$formElementName}\" name=\"{$formElementName}\" value=\"{$value}\">";
        break;
    }
  }
}
?>