<?php
class UserBookingOption {
  private $changeAllowed;
  
  private $booking_id;
  private $event_option_id;
  private $event_group_type;
  private $event_option_type;
  private $event_fee_id;
  private $value_str;
  private $value_num;
  private $value_date;
  private $waitlist;

  /////////////////////////////////////////////////////////////////////////////////////////////////
  // initialize
  private function initialize() {
    foreach ($this as $key => $value)
      $this->$key = null;
  }
  
  public function __construct() {
    $this->initialize();
  }
  
  /////////////////////////////////////////////////////////////////////////////////////////////////
  // security

  /////////////////////////////////////////////////////////////////////////////////////////////////
  // get values
  public function __get($name) {
    if($name == "id") $name = "booking_id";
    if (array_key_exists($name, get_class_vars(get_class())))
      return $this->$name;
      
    return null;
  }

  public function getIDName() {
    return "booking_id";
  }
  
  public function toArray() {
    $array = array();
    foreach ($this as $key => $value)
      $array[$key] = $value; 
    return $array;
  }
  
  public function getValue($convert2Local=0) {
    switch($this->event_group_type) {
      case OPTION_GROUP_FEE:
        return $this->event_fee_id;
        break;
      case OPTION_GROUP_ADDINFO:
        switch($this->event_option_type) {
          case EVENT_OPTION_TEXT:  
          case EVENT_OPTION_TEXT_MULTILINE:
            return $this->value_str;
            break;
          case EVENT_OPTION_DATE:
            if ($convert2Local)
              return CWFTools::dateISO2Local($this->value_date, getenv("LC_ALL"));
            else
              return $this->value_date;
            break;
          case EVENT_OPTION_NUMBER_INT:
            return $this->value_num;
            break;
          case EVENT_OPTION_NUMBER_DEC:
            if ($convert2Local)
              return CWFTools::float2String($this->value_num, getenv("LC_ALL"));
            else
              return $this->value_num;
            break;
        }
        break;
    }
    return null;
  }
  
  /////////////////////////////////////////////////////////////////////////////////////////////////
  // set values
  public function __set($name, $value) {
    if($name == "id") $name = "booking_id";
    if (array_key_exists($name, get_class_vars(get_class())))
      $this->$name = $value;
  }
  
  public function fromArray($array) {
    foreach ($this as $key => &$value) {
      if(isset($array[$key]))
        $value = $array[$key];
    }
  }
  
  /////////////////////////////////////////////////////////////////////////////////////////////////
  // database operations - read

  /////////////////////////////////////////////////////////////////////////////////////////////////
  // database operations - write
  public function writeData(DbConnection $DB, $force=false) {
    if(!isset($_SESSION['admin'])) $force=false;
    if ($this->event_group_type==OPTION_GROUP_FEE && !$this->event_fee_id) {
      if(!isset($_SESSION['booking_id']) || !$_SESSION['booking_id'] || $force)
        $result = $DB->query("DELETE FROM user_booking_options WHERE booking_id='{$this->booking_id}' AND event_option_id='{$this->event_option_id}'");
    }
    else {
      $result = $DB->query("UPDATE user_booking_options
          SET event_fee_id='{$this->event_fee_id}',
              value_str='{$this->value_str}',
              value_num=".(($this->value_num===null) ? "null" : "'{$this->value_num}'").",
              value_date=".(($this->value_date===null) ? "null" : "'{$this->value_date}'").",
              waitlist='".($this->waitlist ? "1" : "0")."'
          WHERE booking_id='{$this->booking_id}'
            AND event_option_id='{$this->event_option_id}';");
      if ($DB->affected_rows()<=0 && ($this->event_fee_id || $this->value_str!=="" || $this->value_num!==null || $this->value_date!==null))
        $result = $DB->query("INSERT INTO user_booking_options(
            booking_id,
            event_option_id,
            event_group_type,
            event_option_type,
            event_fee_id,
            value_str,
            value_num,
            value_date,
            waitlist,
            deleted
          ) VALUES(
            '{$this->booking_id}',
            '{$this->event_option_id}',
            '{$this->event_group_type}',
            '{$this->event_option_type}',
            '{$this->event_fee_id}',
            '{$this->value_str}',
            ".(($this->value_num===null) ? "null" : "'{$this->value_num}'").",
            ".(($this->value_date===null) ? "null" : "'{$this->value_date}'").",
            '".($this->waitlist ? "1" : "0")."',
            0
          );");
    }
    return array($DB->errno(), $DB->error());
  }

  static public function unDeleteData(DbConnection $DB, $id) {
    if(preg_match("~^[0-9]+$~", $id))
      $condition = "event_option_id='{$id}'";
    else
      $condition = $id;
    
    $result = $DB->query("UPDATE user_booking_options SET deleted=0 WHERE $condition;");
    
    if(!$DB->errno() && $DB->affected_rows()>0)
      return array(0, "");
    else 
      return array($DB->errno(), $DB->error());
  }
  
  static public function deleteData(DbConnection $DB, $id, $reallyDelete=0) {
    if(preg_match("~^[0-9]+$~", $id))
      $condition = "event_option_id='{$id}'";
    else
      $condition = $id;
    
    if($reallyDelete)
      $result = $DB->query("DELETE FROM user_booking_options WHERE $condition;");
    else
      $result = $DB->query("UPDATE user_booking_options SET deleted=1 WHERE $condition;");
    
    if(!$DB->errno() && $DB->affected_rows()>0)
      return array(0, "");
    else 
      return array($DB->errno(), $DB->error());
  }

  /////////////////////////////////////////////////////////////////////////////////////////////////
  // database operations - misc
  static public function createTable(DbConnection $DB) {
    $result = $DB->query("
      CREATE TABLE user_booking_options (
        booking_id int unsigned NOT NULL,
        event_option_id int unsigned NOT NULL,
        event_group_type tinyint unsigned,
        event_option_type tinyint unsigned,
        event_fee_id int unsigned,
        value_str text,
        value_num float,
        value_date date,
        waitlist tinyint,
        deleted tinyint,
        PRIMARY KEY(booking_id, event_option_id)
      );");
    if ($result) { echo _("&nbsp;&nbsp;- creating table").": 'user_booking_options'.<br>\n"; }
    else { echo _("&nbsp;&nbsp;<b>- Error when creating table</b>")." 'user_booking_options'<br>\n"; $error = 1; }
  }  
}
?>
