<?php
require_once("cwfUI.inc.php");

class UserBookingUI extends cwfUI {
  public $data;
  protected $error;
  protected $containsAddInfo;
  
  /////////////////////////////////////////////////////////////////////////////////////////////////
  // initialize
  public function __construct(DbConnection $DB, $eventId=null) {
    parent::__construct();
    $this->error = "";
    $this->containsAddInfo = 0;

    if(!$eventId && !isset($_SESSION['event']))
      $this->error = _("No event selected.");
    else {
      $this->data = new UserBooking();
      if($eventId)
        $this->data->setEventId($DB, $eventId);
      else
        $this->data->setEventId($DB, $_SESSION['event']->event_id);

      if(isset($_SESSION['user']) && !$_SESSION['event']->allow_multiple_reg) {
        $this->data->readData($DB);
        if($this->data->booking_id)
          $_SESSION['booking_id'] = $this->data->booking_id;
      }
      else if(isset($_SESSION['registration']['booking_id']) && $_SESSION['registration']['booking_id']) {
        $this->data->readData($DB, "booking_id='{$_SESSION['registration']['booking_id']}'");
      }

      $this->data->setValidation(self::getValidationRules($this->data->form, (bool) $this->data->booking_id));
    }
    
    $this->title = _("Registration");
    $this->header = "  <script type=\"text/javascript\">
    $(document).ready(function(){
      $(\"form\").validate({
        errorClass: 'invalid',\n".
        $this->data->validation->getValidationRulesJQuery("        ")."
      });
    });
  </script>\n";
    if((isset($_POST['kdb_save']) && $_POST['kdb_save']) ||
       (isset($_POST['kdb_submit']) && $_POST['kdb_submit'])) {
      if (isset($_SESSION['registration']['user_id']) && $_SESSION['registration']['user_id'])
        $_POST['user_id'] = $_SESSION['registration']['user_id'];
        
      if (!isset($_SESSION['registration']['booking_id'])) {
        if(isset($_SESSION['user']) || isset($_SESSION['registration']['user_id'])) {
          if($this->saveForm($DB) && !isset($_SESSION['booking_id']))
            $_SESSION['registration']['booking_id'] = $this->data->booking_id;
        }
        else
          $this->data->fromArray($_POST);
      }
    }
    else if (isset($_SESSION['registration']['booking_id']) && isset($_POST['reg_finish']) && $_POST['reg_finish']) {
      unset($_SESSION['registration']['booking_id']);
      if($_SESSION['event']->allow_multiple_reg) {
        $this->data = new UserBooking();
        if($eventId)
          $this->data->setEventId($DB, $eventId);
        else
          $this->data->setEventId($DB, $_SESSION['event']->event_id);
      }
    }
  }
  
  /////////////////////////////////////////////////////////////////////////////////////////////////
  // handle data
  public function getErrorMessage() {
    return $this->error;
  }
  
  static public function getValidationRules(EventForm $form=null, $isChange=false) {
    global $DB;
    
    if(!$form) {
      $form = new EventForm();
      if(isset($_SESSION['event'])) {
        $form->setEventId($_SESSION['event']->event_id);
        $form->readData($DB);
      }
    }

    $rules = array();
    $messages = array();
    foreach ($form->eventForm as $groupId => $group) {
      foreach ($group->eventOptions as $optionId => $option) {
        if(!$isChange || $group->allow_changes) {
          switch($group->group_type) {
            case OPTION_GROUP_FEE:
              if($group->required) {
                if ($group->exclusive) {
                  $rules["event_option_g[{$groupId}]"] = array("required"=>true);
                  $messages["event_option_g[{$groupId}]"] = array("required"=>_("Please select at least one of the options below."));
                }
                else {
                  $rules["event_option[{$option->event_option_id}]"] = array("required"=>true);
                  $messages["event_option[{$option->event_option_id}]"] = array("required"=>_("Please select this option."));
                }
              }
              break;
            case OPTION_GROUP_ADDINFO:
              switch($option->option_type) {
                case EVENT_OPTION_TEXT: 
                case EVENT_OPTION_TEXT_MULTILINE:
                  if($group->required) {
                    $rules["event_option[{$option->event_option_id}]"] = array("required"=>true);
                    $messages["event_option[{$option->event_option_id}]"] = array("required"=>_("This field is mandatory."));
                  }
                  break;
                case EVENT_OPTION_DATE: 
                  if($group->required) {
                    $rules["event_option[{$option->event_option_id}]"] = array("required"=>true, "date_".getenv("LC_ALL")=>true);
                    $messages["event_option[{$option->event_option_id}]"] = array("required"=>_("This field is mandatory."), "date_".getenv("LC_ALL")=>_("Please enter a valid date, e.g. ").CWFTools::dateISO2Local("2000-12-31", getenv("LC_ALL")));
                  }
                  else {
                    $rules["event_option[{$option->event_option_id}]"] = array("date_".getenv("LC_ALL")=>true);
                    $messages["event_option[{$option->event_option_id}]"] = array("date_".getenv("LC_ALL")=>_("Please enter a valid date, e.g. ").CWFTools::dateISO2Local("2000-12-31", getenv("LC_ALL")));
                  }
                  break;
                case EVENT_OPTION_NUMBER_INT:
                  if($group->required) {
                    $rules["event_option[{$option->event_option_id}]"] = array("required"=>true, "digits"=>true);
                    $messages["event_option[{$option->event_option_id}]"] = array("required"=>_("This field is mandatory."), "digits"=>_("Please enter a whole number, e.g. 12"));
                  }
                  else {
                    $rules["event_option[{$option->event_option_id}]"] = array("digits"=>true);
                    $messages["event_option[{$option->event_option_id}]"] = array("digits"=>_("Please enter a whole number, e.g. 12"));
                  }
                  break;
                case EVENT_OPTION_NUMBER_DEC:
                  if($group->required) {
                    $rules["event_option[{$option->event_option_id}]"] = array("required"=>true, "number_int"=>true);
                    $messages["event_option[{$option->event_option_id}]"] = array("required"=>_("This field is mandatory."), "number"=>_("Please enter a (decimal) number, e.g. 12.3"));
                  }
                  else {
                    $rules["event_option[{$option->event_option_id}]"] = array("number_int"=>true);
                    $messages["event_option[{$option->event_option_id}]"] = array("number_int"=>_("Please enter a (decimal) number, e.g. 12.3"));
                  }
                  break;
              }
              break;
          }
        }
      }
    } 

    $validationRules = new FormValidation($rules, $messages);
    return $validationRules;
  }
    
  public function addValidationRules(FormValidation $validationRules) {
    $this->data->validation->addRules($validationRules);

    $this->header = "  <script type=\"text/javascript\">
    $(document).ready(function(){
      $(\"form\").validate({
        errorClass: 'invalid',\n".
        $this->data->validation->getValidationRulesJQuery("        ")."
      });
    });
  </script>\n";
  }
  
  /////////////////////////////////////////////////////////////////////////////////////////////////
  // Database operations
  public function saveForm(DbConnection $DB) {
    if($this->data->fromArray($_POST) && $this->data->user_id) {
      list($errno, $error) = $this->data->writeData($DB);
      if ($errno) {
        $this->error = $error;
        return 0;
      }
      else return 1;
    }
    return 0;
  }
  
  /////////////////////////////////////////////////////////////////////////////////////////////////
  // output
  public function printSubmitButtons() {
    return $this->containsAddInfo;
  }
  
  public function getHTMLView(DbConnection $DB, $allFields=true, $noFormControls=0) {
    if($noFormControls) {
      $this->getHTMLRegistration($DB, $allFields);
    }
    else {
      echo "<form id=\"kdb_form_booking\" action=\"\" method=\"post\" enctype=\"multipart/form-data\">\n";
      echo "<h2 class=\"first\">"._("Register for").": {$_SESSION['event']->event_title}</h2>\n\n";
      echo "<input type=\"hidden\" name=\"user_id\" id=\"user_id\" value=\"{$this->data->user_id}\">\n\n";
      echo "<input type=\"hidden\" name=\"booking_id\" id=\"booking_id\" value=\"{$this->data->booking_id}\">\n\n";
      $this->getHTMLRegistration($DB, $allFields);
?>
<div class="form_controls_bottom">
<p>
  <input type="submit" id="kdb_save2" name="kdb_save" value="<?=_("Save")?>" onClick="kdbSaveEntry('ViewDataUI', 'save', 'kdb_form_booking', refreshHiddenColumns);  return false;">
  <input type="submit" id="kdb_save_close2" name="kdb_save_close" value="<?=_("Save and close")?>" onClick="kdbSaveEntry('ViewDataUI', 'save', 'kdb_form_booking', refreshHiddenColumns); $('#dialog_view').hide(); return false;">
</p>
</div>
<?php
      echo "</form>\n";
    }
  }
  
  public function getHTML(DbConnection $DB) {
    if($this->data->booking_id) {
      echo "<input type=\"hidden\" name=\"booking_id\" id=\"booking_id\" value=\"{$this->data->booking_id}\">\n\n";
      echo "<h3>"._("Event fees")."</h3>\n";
      echo "<table class=\"padding alignleft borderrows\">\n";
      echo "<colgroup>\n";
      echo "  <col style=\"min-width:250px;\"><col>\n";
      echo "<colgroup>\n";
      $grandtotal = 0;
      $first = 1;
      foreach ($this->data->form->eventForm as $groupId => $group) {
        if ($group->group_type==OPTION_GROUP_FEE) {
          echo "<tr>\n";
          if ($first) {
            echo "  <th colspan=\"2\"><h4 class=\"first\">{$group->group_name}</h4>";
            $first=0;
          }
          else
            echo "  <th colspan=\"2\"><h4>{$group->group_name}</h4>";
          echo "</th>\n</tr>\n";
          $bookedOptionsCount = 0;
          foreach ($group->eventOptions as $optionId => $option) {
            if(isset($this->data->bookedOptions[$option->event_option_id])) {
              $bookedOptionsCount++;
              $feeIdx = $this->data->bookedOptions[$option->event_option_id]->getValue();
              $grandtotal += $option->eventFees[$feeIdx]->event_fee;
              echo "<tr>\n";
              echo "  <td>{$option->event_option}</td>\n";
              echo "  <td>".CURRENCY." {$option->eventFees[$feeIdx]->event_fee}</td>\n";
              echo "</tr>\n";
            }
          }
          if($bookedOptionsCount==0) {
              echo "<tr>\n";
              echo "  <td colspan=\"2\">"._("Nothing has been booked in this category")."</td>\n";
              echo "</tr>\n";
            
          }
        }
      }
      echo "<tr>\n";
      echo "  <th><h5>"._("Grand total")."</h5></th>\n";
      echo "  <th><h5>".CURRENCY." {$grandtotal}".($this->data->payment_received ? " ("._("payment received").")" : "")."</h5></th>\n";
      echo "</tr>\n";
      echo "</table>\n\n";
      foreach ($this->data->form->eventForm as $groupId => $group) {
        if ($group->group_type==OPTION_GROUP_ADDINFO) {
          $readonly = "";
          if(!$group->allow_changes)
            $readonly = " readonly";
          else
            $this->containsAddInfo = 1;
          echo "<h3>{$group->group_name}</h3>\n";
          
          foreach ($group->eventOptions as $optionId => $option) {
            echo "<p>\n";
            $this->printAddInfo($group, $option, $readonly);
            echo "</p>\n";
          }
        }
      }
    }
    else {
      $this->getHTMLRegistration($DB);
    }
  }
  
  public function getHTMLRegistration(DbConnection $DB, $all_fields=false) {
    if(!isset($_SESSION['admin'])) $all_fields = false;
    
    foreach ($this->data->form->eventForm as $groupId => $group) {
      echo "<h3>{$group->group_name}";
      if ($group->required && $group->group_type==OPTION_GROUP_FEE && $group->exclusive) {
        if(!$this->data->validation->isValid("event_option_g[{$groupId}]"))
          echo "  <label class=\"invalid\" for=\"event_option_g[$groupId]\" generated=\"true\">".$this->data->validation->getInvalidMessage("event_option_g[{$groupId}]")."</label>";
        else 
          echo "  <label class=\"invalid\" for=\"event_option_g[$groupId]\" generated=\"true\" style=\"display:none;\"></label>";
      }
      echo "</h3>\n";
      foreach ($group->eventOptions as $optionId => $option) {
        echo "<p>\n";
        switch($group->group_type) {
          case OPTION_GROUP_FEE:
            $elementId = "event_option[{$option->event_option_id}]";
            $class = ($group->required ? " class=\"required\"" : "");
            if ($group->exclusive)
              $elementName = "event_option_g[{$groupId}]";
            else
              $elementName = "event_option[{$option->event_option_id}]";
            echo "  <label for=\"$elementId\"$class>{$option->event_option}</label>\n";
            $feeIdx=0;
            /////////////////////////////////////////////////////////////////////////////////
            // form for administration
            if($all_fields) {
              $type = (($group->exclusive) ? "radio" : "checkbox");
              $firstElement=1;
              foreach($option->eventFees as $feeCount => $fee) {
                $checked = ((isset($this->data->bookedOptions[$option->event_option_id]) && $this->data->bookedOptions[$option->event_option_id]->getValue()==$fee->event_fee_id) ? " checked" : "");
                if(!$firstElement) {
                  echo "  <span class=\"indented\">&nbsp;<input type=\"{$type}\" name=\"{$elementName}\" id=\"{$elementId}\" value=\"{$fee->event_fee_id}\"{$checked}></span>\n";
                }
                else {
                  $firstElement=0;
                  echo "  <input type=\"{$type}\" name=\"{$elementName}\" id=\"{$elementId}\" value=\"{$fee->event_fee_id}\"{$checked}>\n";
                }
                if($fee->event_fee) 
                  echo "  <label for=\"{$elementId}\" class=\"fee\">".CURRENCY."&nbsp;{$fee->event_fee}".($fee->deadline ? " (".CWFTools::dateISO2Local($fee->deadline, getenv("LC_ALL")).")" : "")."</label><br>\n";
                else 
                  echo "  <br>\n";
              }
              if($_SESSION['event']->event_id==$this->data->form->event_id && $_SESSION['event']->enable_waitlist) {
                echo "  <span class=\"indented\">&nbsp;<input type=\"checkbox\" name=\"waitlist_{$elementName}\" id=\"waitlist_{$elementId}\" value=\"1\"".
                     ((isset($this->data->bookedOptions[$option->event_option_id]) && $this->data->bookedOptions[$option->event_option_id]->waitlist) ? " checked" : " checked").
                     "></span>\n";
                echo "  <label for=\"waitlist_{$elementId}\" class=\"fee\">"._("waitlist booking")."</label>\n";
              }
            }
            /////////////////////////////////////////////////////////////////////////////////
            // form for users
            else {
              foreach($option->eventFees as $feeCount => $fee) {
                if (!$fee->deadline || $fee->deadline >= date("Y-m-d")) {
                  $feeIdx = $feeCount;
                  break;
                }
              }
              $optionStatus = _('booking deadline exceeded (deadline: ').CWFTools::dateISO2Local($option->eventFees[$feeCount]->deadline, getenv("LC_ALL")).")";

              if ($feeIdx == 0) {
                echo "  <input type=\"".(($group->exclusive) ? "radio" : "checkbox")."\" name=\"{$elementName}\" id=\"{$elementId}\" value=\"\" disabled>\n";
                echo "  <label for=\"{$elementId}\" class=\"fee\">".$optionStatus."</label>\n";
              }
              else {
                $optionStatus = "";
                if ($option->person_limit>0 && $option->person_limit<=$option->no_of_bookings)
                  $optionStatus = _('booked out');
                  
                $type = (($group->exclusive) ? "radio" : "checkbox");
                $checked = ((isset($this->data->bookedOptions[$option->event_option_id]) && $this->data->bookedOptions[$option->event_option_id]->getValue()==$option->eventFees[$feeIdx]->event_fee_id) ? " checked" : "");
                if($optionStatus)
                  echo "  <input type=\"{$type}\" name=\"{$elementName}\" id=\"{$elementId}\" value=\"\" disabled>\n";
                else
                  echo "  <input type=\"{$type}\" name=\"{$elementName}\" id=\"{$elementId}\" value=\"{$option->eventFees[$feeIdx]->event_fee_id}\"{$checked}>\n";
                echo "  <label for=\"{$elementId}\" class=\"fee\">";
                  if($option->eventFees[$feeIdx]->event_fee) 
                    echo CURRENCY."&nbsp;{$option->eventFees[$feeIdx]->event_fee}";
                  if ($optionStatus) echo " ({$optionStatus})";
                  echo "</label>\n";
                if($optionStatus && $_SESSION['event']->enable_waitlist) {
                  echo "<br>\n";
                  echo "  <span class=\"indented\">&nbsp;<input type=\"{$type}\" name=\"{$elementName}\" id=\"{$elementId}\" value=\"{$option->eventFees[$feeIdx]->event_fee_id}\"{$checked}></span>\n";
                  echo "  <label for=\"{$elementId}\" class=\"fee\">"._("waiting list")."</label>\n";
                }
              }
            }
            break;
          case OPTION_GROUP_ADDINFO:
            $this->printAddInfo($group, $option);
            break;
        }
        if (!$group->exclusive) {
          if(!$this->data->validation->isValid("event_option_g[{$groupId}]"))
            echo "  <label class=\"invalid\" for=\"event_option[{$option->event_option_id}]\" generated=\"true\">".$this->data->validation->getInvalidMessage("event_option_g[{$groupId}]")."</label>";
          else 
            echo "  <label class=\"invalid\" for=\"event_option[{$option->event_option_id}]\" generated=\"true\" style=\"display:none;\"></label>";
        }
        echo "</p>\n";
      }
    } 
  }

  private function getClassString($classArray) {
    if(count($classArray)>0)
      return "class=\"".implode(" ", $classArray)."\" ";
    else
      return "";
  }

  private function printAddInfo(EventOptionsGroup $group, EventOption $option, $readonly="") {
    $classes = array();
    if($group->required)
      $classes[] = "required";
    if(!$this->data->validation->isValid("event_option[{$option->event_option_id}]"))
      $classes[] = "error";
    $classes = $this->getClassString($classes);
    echo "  <label {$classes}for=\"event_option_{$option->event_option_id}\">{$option->event_option}:</label>\n";
    if(isset($this->data->bookedOptions[$option->event_option_id]))
      $value=$this->data->bookedOptions[$option->event_option_id]->getValue(1);
    else
      $value="";
    switch($option->option_type) {
      case EVENT_OPTION_TEXT: echo "  <input {$classes}type=\"text\" name=\"event_option[{$option->event_option_id}]\" id=\"event_option[{$option->event_option_id}]\" value=\"{$value}\"{$readonly}>"; break;
      case EVENT_OPTION_TEXT_MULTILINE: echo "  <textarea {$classes}name=\"event_option[{$option->event_option_id}]\" id=\"event_option[{$option->event_option_id}]\"{$readonly}>{$value}</textarea>"; break;
      case EVENT_OPTION_DATE: echo "  <input {$classes}type=\"text\" name=\"event_option[{$option->event_option_id}]\" id=\"event_option[{$option->event_option_id}]\" value=\"{$value}\" placeholder=\"".CWFTools::dateISO2Local("2000-12-31", getenv("LC_ALL"))."\"{$readonly}>"; break;
      case EVENT_OPTION_NUMBER_INT: echo "  <input {$classes}type=\"text\" name=\"event_option[{$option->event_option_id}]\" id=\"event_option[{$option->event_option_id}]\" value=\"{$value}\" placeholder=\""._("Whole number, e.g. ").strval(12)."\"{$readonly}>"; break;
      case EVENT_OPTION_NUMBER_DEC: echo "  <input {$classes}type=\"text\" name=\"event_option[{$option->event_option_id}]\" id=\"event_option[{$option->event_option_id}]\" value=\"{$value}\" placeholder=\""._("Decimal number, e.g. ").CWFTools::float2String(12.3, getenv("LC_ALL"))."\"{$readonly}>"; break;
    }
    if(!$this->data->validation->isValid("event_option[{$option->event_option_id}]"))
      echo "\n  <label for=\"event_option[{$option->event_option_id}]\" class=\"invalid\">{$this->data->validation->getInvalidMessage("event_option[{$option->event_option_id}]")}</label>";
    echo "<br>\n";
  }
}
?>